/*----------------------------------------------------------------------- 
 * 
 * File Name: ZZVectorMultiplyConjugate.c
 * 
 * Author: Creighton, J. D. E.
 * 
 * Revision: $Id: ZZVectorMultiplyConjugate.c,v 1.4 2000/11/09 23:37:22 jolien Exp $
 * 
 *----------------------------------------------------------------------- 
 * 
 * NAME 
 * ZZVectorMultiplyConjugate
 * 
 * SYNOPSIS 
 *
 *
 * DESCRIPTION 
 * Multiply a double-precision complex vector by the complex conjugate
 * of another vector.
 * 
 * DIAGNOSTICS 
 * Null pointer, invalid input size, size mismatch.
 *
 * CALLS
 * 
 * NOTES
 * 
 *----------------------------------------------------------------------- */

#include <math.h>
#include <lal/LALStdlib.h>
#include <lal/VectorOps.h>

NRCSID(ZZVECTORMULTIPLYCONJUGATEC,"$Id: ZZVectorMultiplyConjugate.c,v 1.4 2000/11/09 23:37:22 jolien Exp $");

void
LALZZVectorMultiplyConjugate (
    LALStatus                *status,
    COMPLEX16Vector       *out,
    const COMPLEX16Vector *in1,
    const COMPLEX16Vector *in2
    )
{
  COMPLEX16 *a;
  COMPLEX16 *b;
  COMPLEX16 *c;
  INT4       n;

  INITSTATUS (status, "LALZZVectorMultiplyConjugate", ZZVECTORMULTIPLYCONJUGATEC);

  ASSERT (out, status, VECTOROPS_ENULL, VECTOROPS_MSGENULL);
  ASSERT (in1, status, VECTOROPS_ENULL, VECTOROPS_MSGENULL);

  ASSERT (in2, status, VECTOROPS_ENULL, VECTOROPS_MSGENULL);

  ASSERT (out->data, status, VECTOROPS_ENULL, VECTOROPS_MSGENULL);
  ASSERT (in1->data, status, VECTOROPS_ENULL, VECTOROPS_MSGENULL);
  ASSERT (in2->data, status, VECTOROPS_ENULL, VECTOROPS_MSGENULL);

  ASSERT (out->length > 0, status, VECTOROPS_ESIZE, VECTOROPS_MSGESIZE);
  ASSERT (in1->length == out->length, status,
          VECTOROPS_ESZMM, VECTOROPS_MSGESZMM);
  ASSERT (in2->length == out->length, status,
          VECTOROPS_ESZMM, VECTOROPS_MSGESZMM);

  a = in1->data;
  b = in2->data;
  c = out->data;
  n = out->length;

  while (n-- > 0)
  {
    REAL8 ar = a->re;
    REAL8 ai = a->im;
    REAL8 br = b->re;
    REAL8 bi = b->im;

    c->re = ar*br + ai*bi;
    c->im = ai*br - ar*bi;

    ++a;
    ++b;
    ++c;
  }

  RETURN (status);
}

